@@ -44,9 +44,13 @@ Lastly, a question for Fyodor:  is WSAEHOSTUNREACH a good
 error return for a failed ARP query?  (It should convince
 nmap not to try the host again, while not confusing nmap.)
 
+Update 12/08/04: Dana Epp (dana_at_vulscan.com) sdded SendARP stuff
+for XP firewall (on by default w/SP2)
+
 */
 
 #include "..\tcpip.h"
+#include "..\..\NmapOps.h"
 #include "winip.h"
 
 #define MAXARPTRIES 3
@@ -95,6 +99,7 @@ static int ip2route(const struct in_addr *dest, DWORD *nexthop, DWORD *ifi);
 static void releaseadapter();
 
 static void send_arp(DWORD ifi, DWORD ip);
+static void send_raw_arp(DWORD ifi, DWORD ip);
 static int lookupip(DWORD ip, DWORD ifi);
 
 //	ARP cache
@@ -104,44 +109,14 @@ static int SearchARP(DWORD ip, int ifi, BYTE *phys, int *physlen);
 
 static CRITICAL_SECTION csAdapter, csQueue, csFailCache, csArpCache, csArpTable;
 static HANDLE hEvWakeup, hThread, hSemQueue;
-static killthread = 0;
+static int killthread = 0;
 
-extern struct ops o;
+//	For rawsock fallback
+extern SOCKET global_raw_socket;
+extern int rawsock_avail;
+extern int iphlp_avail;
 
-/* Standard swiped internet checksum routine */
-inline unsigned short in_cksum(unsigned short *ptr,int nbytes) 
-{
-	register int		sum;            /* XXX assumes long == 32 bits */
-	u_short				oddbyte;
-	register u_short	answer;         /* assumes u_short == 16 bits */
-
-/*
- * Our algorithm is simple, using a 32-bit accumulator (sum),
- * we add sequential 16-bit words to it, and at the end, fold back
- * all the carry bits from the top 16 bits into the lower 16 bits.
- */
-
-	sum = 0;
-	while (nbytes > 1)  
-	{
-		sum += *ptr++;
-		nbytes -= 2;
-	}
-/* mop up an odd byte, if necessary */
-	if (nbytes == 1) 
-	{
-		oddbyte = 0;            /* make sure top half is zero */
-		*((u_char *) &oddbyte) = *(u_char *)ptr;   /* one byte only */
-		sum += oddbyte;
-	}
-/*
- * Add back carry outs from top 16 bits to low 16 bits.
- */
-	sum  = (sum >> 16) + (sum & 0xffff);    /* add high-16 to low-16 */
-	sum += (sum >> 16);                     /* add carry */
-	answer = ~sum;          /* ones-complement, then truncate to 16 bits */
-	return(answer);
-}
+extern NmapOps o;
 
 #define SENDQUEUE_LEN 10	//	max outstanding ARP's
 
@@ -232,7 +207,7 @@ static int realsend(LPADAPTER pAdap,
 	if(addrlen != 6)
 		fatal("realsend: non-ethernet address\n");
 
-	build_ethernet(to, from, protocol, packet, len, packetbuf);		
+	build_ethernet(to, from, protocol, (unsigned char *) packet, len, packetbuf);		
 
 	if((lpPacket = PacketAllocatePacket())==NULL)
 	{
@@ -290,21 +265,21 @@ Q_ARP arpcache[ARPCACHELEN];
 int arpfirst = 0;	//	0 <= arpfirst < ARPCACHELEN
 
 PMIB_IPNETTABLE pArpTable;
-int arpalloclen;
+unsigned long arpalloclen;
 int arprefresh = 1;
 
 //	statistics
 static int totaltimetofail = 0;
 static int numfails = 0;
 static int maxfailtime = 0;
-static int queuelen = 0;
+static long queuelen = 0;
 
 //	The actual structure
 static Q_ROUTE sendqueue[SENDQUEUE_LEN];
 static Q_FREE *nextfree = 0;	//	protected by the hSemQueue
 
 //	The send thread
-static DWORD WINAPI SendThreadProc(LPVOID unused0)
+static unsigned int WINAPI SendThreadProc(LPVOID unused0)
 {
 	//	this thread manages send ops
 
@@ -354,7 +329,7 @@ __try {
 				Q_PACKET *p = sendqueue[i].head;
 				BYTE myphys[MAXLEN_PHYSADDR];
 				int myphyslen = MAXLEN_PHYSADDR;
-				int mytype;
+				unsigned long mytype;
 				LPADAPTER pAdap;
 
 #ifdef THREAD_DEBUG
@@ -367,7 +342,7 @@ __try {
 				while(p)
 				{
 					Q_PACKET *next = p->next;
-					realsend(pAdap, p->data, p->len, phys,
+					realsend(pAdap, (char *) p->data, p->len, phys,
 						myphys, myphyslen, mytype, ETH_IP);
 					free(p);
 					p = next;
@@ -549,42 +524,78 @@ begin:
 //	this needs to change for non-Ethernet
 static void send_arp(DWORD ifi, DWORD ip)
 {
-	struct arp_hdr	arp_h;
-	LPADAPTER pAdap;
-	BYTE mymac[6];
-	int len, mytype;
-	struct in_addr myip;
-	BYTE bcastmac[6];	//	more Ethernet code !
-	memset(bcastmac, 0xFF, 6);
+  /* Used to send raw ARP packet on the wire, and then read the result
+     out of the system cache.  Kinda ugly, but anyway ... Windows
+     Firewall (default-on with XP SP2) started ignoring the responses
+     because it didn't send them (reasonable), thus breaking the
+     technique.  Now the SendARP iphlpapi function is used.  A
+     downside is that I believe it blocks, so it may cause
+     local-network ping scans to take much longer on Windows.  I
+     suppose we should send raw and then read results from pcap.  But
+     for now, we have this */
+  HRESULT ret;
+  ULONG uMACAddr[2];
+  ULONG uSize = 6;
+  PBYTE pBuffer;
+  struct in_addr myip;
 
-	if(0 != ifi2ipaddr(ifi, &myip))
-		fatal("sendarp: failed to find my ip ?!?\n");
+  /* For windows95 machines that does not load iphlpapi.dll, send raw
+     ARP packet */ 
+  if( !iphlp_avail )
+    {
+      send_raw_arp(ifi,ip);
+      return;
+    } 
 
-	//	get the MAC et al
-	len = 6;
-	pAdap = if2adapter(ifi, mymac, &len, &mytype);
-	if(!pAdap)
-	{
-		//	do nothing for localhost scan
-		if(myip.s_addr == 0x0100007f) return;
-		else fatal("send_arp: can't send on this interface\n");
-	}
+  ret = SendARP( ip, 0, uMACAddr, &uSize );
 
-	arp_h.ar_hrd=0x0100;
+  if( NO_ERROR == ret )
+    {
+      pBuffer = (PBYTE)uMACAddr;
+      AddToARPCache( ip, ifi, pBuffer, (int)uSize );
+    } 
+} 
 
-	arp_h.ar_pro=0x0008;                    /* format of protocol address */
-	arp_h.ar_hln=6;                         /* length of hardware address */
+//      this to send raw arp packet 
+static void send_raw_arp(DWORD ifi, DWORD ip)
+{
+        struct arp_hdr  arp_h;
+        LPADAPTER pAdap;
+        BYTE mymac[6];
+        int len;
+        unsigned long mytype;
+        struct in_addr myip;
+        BYTE bcastmac[6];       //      more Ethernet code !
+        memset(bcastmac, 0xFF, 6);
+
+        if(0 != ifi2ipaddr(ifi, &myip))
+                fatal("sendarp: failed to find my ip ?!?\n");
+
+        //      get the MAC et al
+        len = 6;
+        pAdap = if2adapter(ifi, mymac, &len, &mytype);
+        if(!pAdap)
+        {
+                //      do nothing for localhost scan
+                if(myip.s_addr == 0x0100007f) return;
+                else fatal("send_arp: can't send on this interface\n");
+        }
+
+        arp_h.ar_hrd=0x0100;
+
+        arp_h.ar_pro=0x0008;                    /* format of protocol address */
+        arp_h.ar_hln=6;                         /* length of hardware address */
     arp_h.ar_pln=4;                         /* length of protocol addres */
     arp_h.ar_op=0x0100 ;
-	memcpy(arp_h.ar_sha,mymac,6);
-	memcpy(arp_h.ar_spa,&myip.s_addr,4);
-	memset(arp_h.ar_tha,0,6);
-	memcpy(arp_h.ar_tpa,&ip,4);
+        memcpy(arp_h.ar_sha,mymac,6);
+        memcpy(arp_h.ar_spa,&myip.s_addr,4);
+        memset(arp_h.ar_tha,0,6);
+        memcpy(arp_h.ar_tpa,&ip,4);
 
-	realsend(pAdap, (char*)&arp_h, sizeof(arp_h),
-		bcastmac, mymac, len, mytype, ETH_ARP);
+        realsend(pAdap, (char*)&arp_h, sizeof(arp_h),
+                bcastmac, mymac, len, mytype, ETH_ARP);
 
-	releaseadapter();
+        releaseadapter();
 }
 
 //	resolves an ip addr into a nexthop and index
@@ -744,6 +755,17 @@ static void releaseadapter()
 	LeaveCriticalSection(&csAdapter);
 }
 
+static int fallback_raw_send(const char *packet, int len, 
+	   struct sockaddr *to, int tolen) 
+{
+	if(!rawsock_avail) {
+		fatal("fallback_raw_send: no raw sockets\n"
+			"This means that you tried to send to an unsupported interface.\n");
+	}
+
+	return sendto(global_raw_socket, packet, len, 0, to, tolen);
+}
+
 //	The almighty pcapsendraw
 //	This is the whole point of this file :)
 int pcapsendraw(const char *packet, int len, 
@@ -753,14 +775,15 @@ int pcapsendraw(const char *packet, int len,
 	int cb = 0;
 	int nRes, i;
 	DWORD nextip;
-	int ifi;
+	unsigned long ifi;
 	LPADAPTER pAdap;
 	BYTE myphys[MAXLEN_PHYSADDR], tphys[MAXLEN_PHYSADDR];
 	int physlen = MAXLEN_PHYSADDR;
 	DWORD type;
+	const WINIP_IF *target_ifentry;
 
 	if(!pcapsend_inited)
-		fatal("pcapsendraw: pcapsend not initialized\n");
+		return fallback_raw_send(packet, len, to, tolen);
 
 	if(-1 == ip2route(&sin->sin_addr, &nextip, &ifi))
 	{
@@ -768,6 +791,10 @@ int pcapsendraw(const char *packet, int len,
 		return -1;	//	no route to host
 	}
 
+	target_ifentry = ifi2ifentry(ifi);
+	if(!target_ifentry || !target_ifentry->pcapname)
+		return fallback_raw_send(packet, len, to, tolen);
+
 	//	check the failcache
 	EnterCriticalSection(&csFailCache);
 	for(i = 0; i < FAILCACHELEN; i++)
@@ -806,7 +833,7 @@ int pcapsendraw(const char *packet, int len,
 void pcapsend_init()
 {
 	int i, nRes;
-	DWORD id;
+	unsigned int id;
 
 	if(pcapsend_inited) return;
 	pcapsend_inited = 1;
@@ -838,7 +865,7 @@ void pcapsend_init()
 	nRes = GetIpNetTableSafe(pArpTable, &arpalloclen, FALSE);
 	if(arpalloclen == 0)
 	{
-		if(o.debugging)
+		if(o.debugging && nRes != ERROR_NO_DATA)
 			printf("ARP table length failure (%lu) during init -- try kludge1 :(\n", nRes);
 		arpalloclen = 100 * sizeof(MIB_IPNETROW) + 8;
 	}
@@ -855,7 +882,7 @@ void pcapsend_init()
 
 	if(nRes != NO_ERROR)
 	{
-		if(o.debugging)
+		if(o.debugging && nRes != ERROR_NO_DATA)
 			printf("ARP failure (%lu) during init -- trying kludge2\n", nRes);
 		pArpTable->dwNumEntries = 0;
 	}
@@ -977,7 +1004,7 @@ tryagain:
 	else
 	{
 		PMIB_IPFORWARDTABLE pTable = 0;
-		int cb = 0;
+		unsigned long cb = 0;
 		int bestmatch = -1;
 		int bestmask, bestmetric;
 		int nRes, i;
@@ -1057,7 +1084,7 @@ pass1:
 	if(arprefresh)
 	{
 		//	refresh
-		int len = arpalloclen;
+		unsigned long len = arpalloclen;
 
 #ifdef THREAD_DEBUG
 		printf("lookupip: refreshing ARP table\n");
