@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap.cc,v 1.53 2004/08/31 03:46:19 fyodor Exp $ */
+/* $Id: nmap.cc,v 1.61 2005/02/07 08:52:52 fyodor Exp $ */
 
 #include "nmap.h"
 #include "osscan.h"
@@ -223,6 +223,9 @@ int nmap_main(int argc, char *argv[]) {
   Target *currenths;
   vector<Target *> Targets;
   char *proberr;
+#if WIN32
+  bool skip_winip_init = false;
+#endif
   char emptystring[1];
   int sourceaddrwarning = 0; /* Have we warned them yet about unguessable
 				source addresses? */
@@ -253,6 +256,7 @@ int nmap_main(int argc, char *argv[]) {
       {"scanflags", required_argument, 0, 0},
       {"host_timeout", required_argument, 0, 0},
       {"scan_delay", required_argument, 0, 0},
+      {"max_scan_delay", required_argument, 0, 0},
       {"oA", required_argument, 0, 0},  
       {"oN", required_argument, 0, 0},
       {"oM", required_argument, 0, 0},  
@@ -271,8 +275,12 @@ int nmap_main(int argc, char *argv[]) {
       {"version_trace", no_argument, 0, 0}, /* Display -sV related activity */
       {"fuzzy", no_argument, 0, 0}, /* Alias for osscan_guess */
       {"data_length", required_argument, 0, 0},
+      {"stylesheet", required_argument, 0, 0},
+      {"no-stylesheet", no_argument, 0, 0},
       {"rH", no_argument, 0, 0},
       {"vv", no_argument, 0, 0},
+      {"ff", no_argument, 0, 0},
+      {"mtu", required_argument, 0, 0},
       {"append_output", no_argument, 0, 0},
       {"noninteractive", no_argument, 0, 0},
       {"ttl", required_argument, 0, 0}, /* Time to live */
@@ -285,6 +293,7 @@ int nmap_main(int argc, char *argv[]) {
       {"win_noiphlpapi", no_argument, 0, 0}, 
       {"win_help", no_argument, 0, 0},
       {"win_trace", no_argument, 0, 0},
+      {"win_skip_winip_init", no_argument, 0, 0},
 #endif
       {0, 0, 0, 0}
     };
@@ -308,7 +317,7 @@ int nmap_main(int argc, char *argv[]) {
     case 0:
       if (strcmp(long_options[option_index].name, "max_rtt_timeout") == 0) {
 	o.setMaxRttTimeout(atoi(optarg));
-	if (o.maxRttTimeout() <= 5) {
+	if (o.maxRttTimeout() < 5) {
 	  fatal("max_rtt_timeout is given in milliseconds and must be at least 5");
 	}       
         if (o.maxRttTimeout() < 20) {
@@ -377,6 +386,8 @@ int nmap_main(int argc, char *argv[]) {
 	wo.noiphlpapi = 1; 
       } else if (strcmp(long_options[option_index].name, "win_trace") == 0 ) { 
 	wo.trace++; 
+      } else if (strcmp(long_options[option_index].name, "win_skip_winip_init") == 0 ) { 
+	skip_winip_init = true;
       } else if (strcmp(long_options[option_index].name, "win_help") == 0 ) { 
 	printf("Windows-specific options:\n\n"); 
 	printf(" --win_list_interfaces : list all network interfaces\n"); 
@@ -396,8 +407,17 @@ int nmap_main(int argc, char *argv[]) {
 	o.scan_delay = atoi(optarg);
 	if (o.scan_delay <= 0) {
 	  fatal("scan_delay must be greater than 0");
-	}   
+	}
+	if (o.scan_delay > o.maxTCPScanDelay()) o.setMaxTCPScanDelay(o.scan_delay);
+	if (o.scan_delay > o.maxUDPScanDelay()) o.setMaxUDPScanDelay(o.scan_delay);
 	o.max_parallelism = 1;
+      } else if (strcmp(long_options[option_index].name, "max_scan_delay") == 0) {
+	unsigned int scand = atoi(optarg);
+	if (scand < 0) {
+	  fatal("max_scan_delay must be greater than 0");
+	}
+	o.setMaxTCPScanDelay(scand);
+	o.setMaxUDPScanDelay(scand);
       } else if (strcmp(long_options[option_index].name, "randomize_hosts") == 0
 		 || strcmp(long_options[option_index].name, "rH") == 0) {
 	o.randomize_hosts = 1;
@@ -420,6 +440,10 @@ int nmap_main(int argc, char *argv[]) {
 	  o.extra_payload = (char *) safe_malloc(o.extra_payload_length);
 	  get_random_bytes(o.extra_payload, o.extra_payload_length);
 	}
+      } else if (strcmp(long_options[option_index].name, "stylesheet") == 0) {
+	o.setXSLStyleSheet(optarg);
+      } else if (strcmp(long_options[option_index].name, "no-stylesheet") == 0) {
+	o.setXSLStyleSheet(NULL);
       } else if (strcmp(long_options[option_index].name, "oN") == 0) {
 	normalfilename = optarg;
       } else if (strcmp(long_options[option_index].name, "oG") == 0 ||
@@ -464,6 +488,12 @@ int nmap_main(int argc, char *argv[]) {
       } else if (strcmp(long_options[option_index].name, "vv") == 0) {
 	/* Compatability hack ... ugly */
 	o.verbose += 2;
+      } else if (strcmp(long_options[option_index].name, "ff") == 0) {
+	o.fragscan += 16; 
+      } else if (strcmp(long_options[option_index].name, "mtu") == 0) {
+        o.fragscan = atoi(optarg);
+        if (o.fragscan <= 0 || o.fragscan % 8 != 0)
+            fatal("Data payload MTU must be >0 and multiple of 8");
       } else {
 	fatal("Unknown long option (%s) given@#!$#$", long_options[option_index].name);
       }
@@ -520,7 +550,7 @@ int nmap_main(int argc, char *argv[]) {
     case 'e': 
       strncpy(o.device, optarg,63); o.device[63] = '\0'; break;
     case 'F': fastscan++; break;
-    case 'f': o.fragscan++; break;
+    case 'f': o.fragscan += 8; break;
     case 'g': 
       o.magic_port = atoi(optarg);
       o.magic_port_set = 1;
@@ -626,7 +656,7 @@ int nmap_main(int argc, char *argv[]) {
       } else if (*optarg == 'O') {
 	fatal("-PO (the letter O)? No such option. Perhaps you meant to disable pings with -P0 (Zero).");
       } else { 
-	fatal("Illegal Argument to -P, use -P0, -PI, -PB, -PM, -PP, -PT, or -PT80 (or whatever number you want for the TCP probe destination port)"); 
+	fatal("Illegal Argument to -P, use -P0, -PI, -PB, -PE, -PM, -PP, -PA, -PU, -PT, or -PT80 (or whatever number you want for the TCP probe destination port)"); 
       }
       break;
     case 'p': 
@@ -702,12 +732,14 @@ int nmap_main(int argc, char *argv[]) {
 	o.setMinRttTimeout(100);
 	o.setMaxRttTimeout(1250);
 	o.setInitialRttTimeout(500);
+        o.setMaxTCPScanDelay(10);
       } else if (*optarg == '5' || (strcasecmp(optarg, "Insane") == 0)) {
 	o.timing_level = 5;
 	o.setMinRttTimeout(50);
 	o.setMaxRttTimeout(300);
 	o.setInitialRttTimeout(250);
 	o.host_timeout = 900000;
+        o.setMaxTCPScanDelay(5);
       } else {
 	fatal("Unknown timing mode (-T argment).  Use either \"Paranoid\", \"Sneaky\", \"Polite\", \"Normal\", \"Aggressive\", \"Insane\" or a number from 0 (Paranoid) to 5 (Insane)");
       }
@@ -721,7 +753,8 @@ int nmap_main(int argc, char *argv[]) {
   }
 
 #ifdef WIN32
-  winip_postopt_init();
+  if (!skip_winip_init)
+    winip_postopt_init();
 #endif
 
 #if HAVE_SIGNAL
@@ -844,13 +877,21 @@ int nmap_main(int argc, char *argv[]) {
 		ftp.server_name, inet_ntoa(ftp.server)); 
   }
   fflush(stdout);
+  fflush(stderr);
 
   timep = time(NULL);
   
   /* Brief info incase they forget what was scanned */
   Strncpy(mytime, ctime(&timep), sizeof(mytime));
   chomp(mytime);
-  log_write(LOG_XML, "<?xml version=\"1.0\" ?>\n<!-- ");
+  char *xslfname = o.XSLStyleSheet();
+  char xslline[1024];
+  if (xslfname) {
+    char *p = xml_convert(xslfname);
+    snprintf(xslline, sizeof(xslline), "<?xml-stylesheet href=\"%s\" type=\"text/xsl\"?>\n", p);
+    free(p);
+  }  else xslline[0] = '\0';
+  log_write(LOG_XML, "<?xml version=\"1.0\" ?>\n%s<!-- ", xslline);
   log_write(LOG_NORMAL|LOG_MACHINE, "# ");
   log_write(LOG_NORMAL|LOG_MACHINE|LOG_XML, "%s %s scan initiated %s as: ", NMAP_NAME, NMAP_VERSION, mytime);
   
@@ -867,8 +908,8 @@ int nmap_main(int argc, char *argv[]) {
   for(i=0; i < argc; i++) 
     log_write(LOG_XML, (i == argc-1)? "%s\" " : "%s ", fakeargv[i]);
 
-  log_write(LOG_XML, "start=\"%lu\" version=\"%s\" xmloutputversion=\"1.01\">\n",
-	    (unsigned long) timep, NMAP_VERSION);
+  log_write(LOG_XML, "start=\"%lu\" startstr=\"%s\" version=\"%s\" xmloutputversion=\"1.01\">\n",
+	    (unsigned long) timep, mytime, NMAP_VERSION);
 
   output_xml_scaninfo_records(ports);
 
@@ -1014,85 +1055,88 @@ int nmap_main(int argc, char *argv[]) {
     
       /* I used to check that !currenths->wierd_responses, but in some
 	 rare cases, such IPs CAN be port successfully scanned and even connected to */
-      if (currenths->flags & HOST_UP) {
-	if (o.af() == AF_INET && o.RawScan()) { 
-	  if (currenths->SourceSockAddr(NULL, NULL) != 0) {
-	    if (o.SourceSockAddr(&ss, &sslen) == 0) {
-	      currenths->setSourceSockAddr(&ss, sslen);
-	    } else {
-	      if (gethostname(myname, MAXHOSTNAMELEN) || 
-		  resolve(myname, &ss, &sslen, o.af()) == 0)
-		fatal("Cannot get hostname!  Try using -S <my_IP_address> or -e <interface to scan through>\n"); 
+      if (!(currenths->flags & HOST_UP)) {
+	delete currenths;
+	continue;
+      }
+
+      if (o.af() == AF_INET && o.RawScan()) { 
+	if (currenths->SourceSockAddr(NULL, NULL) != 0) {
+	  if (o.SourceSockAddr(&ss, &sslen) == 0) {
+	    currenths->setSourceSockAddr(&ss, sslen);
+	  } else {
+	    if (gethostname(myname, MAXHOSTNAMELEN) || 
+		resolve(myname, &ss, &sslen, o.af()) == 0)
+	      fatal("Cannot get hostname!  Try using -S <my_IP_address> or -e <interface to scan through>\n"); 
 	    
-	      o.setSourceSockAddr(&ss, sslen);
-	      currenths->setSourceSockAddr(&ss, sslen);
-	      if (! sourceaddrwarning) {
-		fprintf(stderr, "WARNING:  We could not determine for sure which interface to use, so we are guessing %s .  If this is wrong, use -S <my_IP_address>.\n", inet_socktop(&ss));
-		sourceaddrwarning = 1;
-	      }
+	    o.setSourceSockAddr(&ss, sslen);
+	    currenths->setSourceSockAddr(&ss, sslen);
+	    if (! sourceaddrwarning) {
+	      fprintf(stderr, "WARNING:  We could not determine for sure which interface to use, so we are guessing %s .  If this is wrong, use -S <my_IP_address>.\n", inet_socktop(&ss));
+	      sourceaddrwarning = 1;
 	    }
 	  }
-	  if (!*currenths->device)
-	    if (ipaddr2devname( currenths->device, currenths->v4sourceip()) != 0)
-	      fatal("Could not figure out what device to send the packet out on!  You might possibly want to try -S (but this is probably a bigger problem).  If you are trying to sp00f the source of a SYN/FIN scan with -S <fakeip>, then you must use -e eth0 (or other devicename) to tell us what interface to use.\n");
-	
-	  /* Groups should generally use the same device as properties
-	     change quite a bit between devices.  Plus dealing with a
-	     multi-device group can be a pain programmatically. So if
-	     this Target has a different device the rest, we give it
-	     back. */
-	  if (Targets.size() > 0 && strcmp(Targets[Targets.size() - 1]->device, currenths->device)) {
-	    returnhost(hstate);
-	    numhosts_scanned--; numhosts_up--;
-	    break;
-	  }
-	  o.decoys[o.decoyturn] = currenths->v4source();
 	}
-	Targets.push_back(currenths);
+	if (!*currenths->device)
+	  if (ipaddr2devname( currenths->device, currenths->v4sourceip()) != 0)
+	    fatal("Could not figure out what device to send the packet out on!  You might possibly want to try -S (but this is probably a bigger problem).  If you are trying to sp00f the source of a SYN/FIN scan with -S <fakeip>, then you must use -e eth0 (or other devicename) to tell us what interface to use.\n");
+	
+	/* Groups should generally use the same device as properties
+	   change quite a bit between devices.  Plus dealing with a
+	   multi-device group can be a pain programmatically. So if
+	   this Target has a different device the rest, we give it
+	   back. */
+	if (Targets.size() > 0 && strcmp(Targets[Targets.size() - 1]->device, currenths->device)) {
+	  returnhost(hstate);
+	  numhosts_scanned--; numhosts_up--;
+	  break;
+	}
+	o.decoys[o.decoyturn] = currenths->v4source();
       }
+      Targets.push_back(currenths);
     }
-  
+    
     if (Targets.size() == 0)
       break; /* Couldn't find any more targets */
-
+    
     // Our source must be set in decoy list because nexthost() call can
     // change it (that issue really should be fixed when possible)
     if (o.af() == AF_INET && o.RawScan())
       o.decoys[o.decoyturn] = Targets[0]->v4source();
-
+    
     /* I now have the group for scanning in the Targets vector */
     /* TODO: Add parallel-capable scans here */
     if (o.synscan)
       ultra_scan(Targets, ports, SYN_SCAN);
-
+    
     if (o.ackscan)
       ultra_scan(Targets, ports, ACK_SCAN);
-
+    
     if (o.windowscan)
       ultra_scan(Targets, ports, WINDOW_SCAN);
-
+    
     if (o.finscan)
       ultra_scan(Targets, ports, FIN_SCAN);
-
+    
     if (o.xmasscan)
       ultra_scan(Targets, ports, XMAS_SCAN);
-
+    
     if (o.nullscan)
       ultra_scan(Targets, ports, NULL_SCAN);
-
+    
     if (o.maimonscan)
       ultra_scan(Targets, ports, MAIMON_SCAN);
-
+    
     if (o.udpscan)
       ultra_scan(Targets, ports, UDP_SCAN);
-
+    
     if (o.connectscan)
       ultra_scan(Targets, ports, CONNECT_SCAN);
-
+    
     if (o.ipprotscan)
       ultra_scan(Targets, ports, IPPROT_SCAN);
-
-   /* These lame functions can only handle one target at a time */
+    
+    /* These lame functions can only handle one target at a time */
     for(targetno = 0; targetno < Targets.size(); targetno++) {
       currenths = Targets[targetno];
       if (o.idlescan) idle_scan(currenths, ports->tcp_ports, 
@@ -1765,6 +1809,7 @@ char *statenum2str(int state) {
   case PORT_UNFILTERED: return "UNfiltered"; break;
   case PORT_CLOSED: return "closed"; break;
   case PORT_OPENFILTERED: return "open|filtered"; break;
+  case PORT_CLOSEDFILTERED: return "closed|filtered"; break;
   default: return "unknown"; break;
   }
   return "unknown";
@@ -1908,6 +1953,7 @@ void sigdie(int signo) {
     break;
   }
   fflush(stdout);
+  fflush(stderr);
   log_close(LOG_MACHINE|LOG_NORMAL|LOG_SKID);
   if (abt) abort();
   exit(1);
@@ -1940,7 +1986,7 @@ int nmap_fetchfile(char *filename_returned, int bufferlen, char *file) {
     }
   }
 
-  if ((dirptr = getenv("NMAPDIR"))) {
+  if (!foundsomething && (dirptr = getenv("NMAPDIR"))) {
     res = snprintf(filename_returned, bufferlen, "%s/%s", dirptr, file);
     if (res > 0 && res < bufferlen) {
       if (fileexistsandisreadable(filename_returned))
@@ -1960,7 +2006,7 @@ int nmap_fetchfile(char *filename_returned, int bufferlen, char *file) {
     if (!foundsomething && getuid() != geteuid()) {
       pw = getpwuid(geteuid());
       if (pw) {
-	res = snprintf(filename_returned, bufferlen, "%s/nmap/%s", pw->pw_dir, file);
+	res = snprintf(filename_returned, bufferlen, "%s/.nmap/%s", pw->pw_dir, file);
 	if (res > 0 && res < bufferlen) {
 	  if (fileexistsandisreadable(filename_returned))
 	    foundsomething = 1;
@@ -1997,11 +2043,11 @@ int nmap_fetchfile(char *filename_returned, int bufferlen, char *file) {
     if (res > 0 && res < bufferlen) {
       if (fileexistsandisreadable(dot_buffer)) {
 #ifdef WIN32
-	if (warningcount++ < 5 && o.debugging)
+	if (warningcount++ < 1 && o.debugging)
 #else
-	if(warningcount++ < 5)
+	if(warningcount++ < 1)
 #endif
-	  error("WARNING!  The following files exist and are readable: %s and %s.  I am choosing %s for security reasons.  set NMAPDIR=. to give priority to files in your local directory", filename_returned, dot_buffer, filename_returned);
+	  error("Warning: File %s exists, but Nmap is using %s for security and consistency reasons.  set NMAPDIR=. to give priority to files in your local directory (may affect the other data files too).", dot_buffer, filename_returned);
       }
     }
   }
