@@ -1,4 +1,3 @@
-
 /***************************************************************************
  * nsock_core.c -- This contains the core engine routines for the nsock    *
  * parallel socket event library.                                          *
@@ -54,7 +53,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_core.c,v 1.21 2004/08/29 09:12:05 fyodor Exp $ */
+/* $Id: nsock_core.c,v 1.23 2005/01/31 20:40:45 fyodor Exp $ */
 
 #include "nsock_internal.h"
 #include "gh_list.h"
@@ -96,6 +95,7 @@ static int wait_for_events(mspool *ms, int msec_timeout) {
   int event_msecs; /* Msecs before an event goes off */
   int combined_msecs;
   int sock_err = 0;
+  int socketno = 0;
   struct timeval select_tv;
   struct timeval *select_tv_p;
 
@@ -131,16 +131,37 @@ static int wait_for_events(mspool *ms, int msec_timeout) {
       select_tv_p = NULL;
     }
     
-    
-    /* Set up the descriptors for select */
-    ms->mioi.fds_results_r = ms->mioi.fds_master_r;
-    ms->mioi.fds_results_w = ms->mioi.fds_master_w;
-    ms->mioi.fds_results_x = ms->mioi.fds_master_x;
-    
-    ms->mioi.results_left = select(ms->mioi.max_sd + 1, &ms->mioi.fds_results_r, &ms->mioi.fds_results_w, &ms->mioi.fds_results_x, select_tv_p);
-    gettimeofday(&nsock_tod, NULL); /* Who knows how long select sat around for */
+    /* Figure out whether there are any FDs in the sets, as @$@!$#
+       Windows returns WSAINVAL (10022) if you call a select() with no
+       FDs, even though the Linux man page says that doing so is a
+       good, reasonably portable way to sleep with subsecond
+       precision.  Sigh */
+    for(socketno = ms->mioi.max_sd; socketno >= 0; socketno--) {
+      if(FD_ISSET(socketno, &ms->mioi.fds_master_r) ||
+	 FD_ISSET(socketno, &ms->mioi.fds_master_w) ||
+	 FD_ISSET(socketno, &ms->mioi.fds_master_x))
+	break;
+      else ms->mioi.max_sd--;
+    }
 
-  } while (ms->mioi.results_left == -1 && (sock_err = socket_errno()) == EINTR);
+    if (ms->mioi.max_sd < 0) {
+      ms->mioi.results_left = 0;
+      if (combined_msecs > 0)
+	usleep(combined_msecs * 1000);
+    } else {
+
+      /* Set up the descriptors for select */
+      ms->mioi.fds_results_r = ms->mioi.fds_master_r;
+      ms->mioi.fds_results_w = ms->mioi.fds_master_w;
+      ms->mioi.fds_results_x = ms->mioi.fds_master_x;
+      
+      ms->mioi.results_left = select(ms->mioi.max_sd + 1, &ms->mioi.fds_results_r, &ms->mioi.fds_results_w, &ms->mioi.fds_results_x, select_tv_p);
+      if (ms->mioi.results_left == -1)
+	sock_err = socket_errno();
+    }
+
+    gettimeofday(&nsock_tod, NULL); /* Due to usleep or select delay */
+  } while (ms->mioi.results_left == -1 && sock_err == EINTR);
   
   if (ms->mioi.results_left == -1 && sock_err != EINTR) {
     ms->errnum = sock_err;
@@ -426,7 +447,7 @@ static int do_actual_read(mspool *ms, msevent *nse) {
   char buf[8192];
   int buflen = 0;
   msiod *iod = nse->iod;
-  int err;
+  int err = 0;
   int max_chunk = NSOCK_READ_CHUNK_SIZE;
   int startlen = FILESPACE_LENGTH(&nse->iobuf);
 
@@ -435,30 +456,32 @@ static int do_actual_read(mspool *ms, msevent *nse) {
 
   if (!iod->ssl) {
     /* Traditional read() - no SSL - using recv() because that works better on Windows */
-    while(((buflen = recv(iod->sd, buf, sizeof(buf), 0)) > 0) ||
-	  ( buflen == -1 && socket_errno() == EINTR)) {
-      if (fscat(&nse->iobuf, buf, buflen) == -1) {
-	nse->event_done = 1;
-	nse->status = NSE_STATUS_ERROR;
-	nse->errnum = ENOMEM;
-	return -1;
+    do {
+      buflen = recv(iod->sd, buf, sizeof(buf), 0);
+      if (buflen == -1) err = socket_errno();
+      if (buflen > 0) {
+	if (fscat(&nse->iobuf, buf, buflen) == -1) {
+	  nse->event_done = 1;
+	  nse->status = NSE_STATUS_ERROR;
+	  nse->errnum = ENOMEM;
+	  return -1;
+	}
+
+	// Sometimes a service just spews and spews data.  So we return
+	// after a somewhat large amount to avoid monopolizing resources
+	// and avoid DOS attacks.
+	if (FILESPACE_LENGTH(&nse->iobuf) > max_chunk)
+	  return FILESPACE_LENGTH(&nse->iobuf) - startlen;
+	
+	// No good reason to read again if we we were successful in the read but
+	// didn't fill up the buffer.  I'll insist on it being TCP too, because I
+	// think UDP might only give me one packet worth at a time (I dunno ...).
+	if (buflen > 0 && buflen < sizeof(buf) && iod->lastproto == IPPROTO_TCP)
+	  return FILESPACE_LENGTH(&nse->iobuf) - startlen;
       }
-      
-      // Sometimes a service just spews and spews data.  So we return
-      // after a somewhat large amount to avoid monopolizing resources
-      // and avoid DOS attacks.
-      if (FILESPACE_LENGTH(&nse->iobuf) > max_chunk)
-	return FILESPACE_LENGTH(&nse->iobuf) - startlen;
-      
-      // No good reason to read again if we we were successful in the read but
-      // didn't fill up the buffer.  I'll insist on it being TCP too, because I
-      // think UDP might only give me one packet worth at a time (I dunno ...).
-      if (buflen > 0 && buflen < sizeof(buf) && iod->lastproto == IPPROTO_TCP)
-	return FILESPACE_LENGTH(&nse->iobuf) - startlen;
-    }
+    } while (buflen > 0 || (buflen == -1 && err == EINTR));
 
     if (buflen == -1) {
-      err = socket_errno();
       if (err != EINTR && err != EAGAIN) {
 	nse->event_done = 1;
 	nse->status = NSE_STATUS_ERROR;
@@ -928,7 +951,7 @@ void nsock_trace_handler_callback(mspool *ms, msevent *nse) {
   nsi = nse->iod;
 
   if (nse->status == NSE_STATUS_ERROR) {
-    snprintf(errstr, sizeof(errstr), "[%s] ", strerror(nse->errnum));
+    snprintf(errstr, sizeof(errstr), "[%s (%d)] ", strerror(nse->errnum), nse->errnum);
   } else errstr[0] = '\0';
 
   // Some types have special tracing treatment
