@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: service_scan.cc,v 1.35 2004/08/29 09:12:03 fyodor Exp $ */
+/* $Id: service_scan.cc,v 1.38 2004/11/06 03:41:53 fyodor Exp $ */
 
 
 #include "service_scan.h"
@@ -822,20 +822,20 @@ void ServiceProbe::setProbeDetails(char *pd, int lineno) {
   char delimiter;
 
   if (!pd || !*pd)
-    fatal("Parse error on line %d of nmap-services-probes: no arguments found!", lineno);
+    fatal("Parse error on line %d of nmap-service-probes: no arguments found!", lineno);
 
   // First the protocol
   if (strncmp(pd, "TCP ", 4) == 0)
       probeprotocol = IPPROTO_TCP;
   else if (strncmp(pd, "UDP ", 4) == 0)
       probeprotocol = IPPROTO_UDP;
-  else fatal("Parse error on line %d of nmap-services-probes: invalid protocol", lineno);
+  else fatal("Parse error on line %d of nmap-service-probes: invalid protocol", lineno);
   pd += 4;
 
   // Next the service name
-  if (!isalnum(*pd)) fatal("Parse error on line %d of nmap-services-probes - bad probe name", lineno);
+  if (!isalnum(*pd)) fatal("Parse error on line %d of nmap-service-probes - bad probe name", lineno);
   p = strchr(pd, ' ');
-  if (!p) fatal("Parse error on line %d of nmap-services-probes - nothing after probe name", lineno);
+  if (!p) fatal("Parse error on line %d of nmap-service-probes - nothing after probe name", lineno);
   len = p - pd;
   probename = (char *) safe_malloc(len + 1);
   memcpy(probename, pd, len);
@@ -844,13 +844,13 @@ void ServiceProbe::setProbeDetails(char *pd, int lineno) {
   // Now for the probe itself
   pd = p+1;
 
-  if (*pd != 'q') fatal("Parse error on line %d of nmap-services-probes - probe string must begin with 'q'", lineno);
+  if (*pd != 'q') fatal("Parse error on line %d of nmap-service-probes - probe string must begin with 'q'", lineno);
   delimiter = *(++pd);
   p = strchr(++pd, delimiter);
-  if (!p) fatal("Parse error on line %d of nmap-services-probes -- no ending delimiter for probe string", lineno);
+  if (!p) fatal("Parse error on line %d of nmap-service-probes -- no ending delimiter for probe string", lineno);
   *p = '\0';
   if (!cstring_unescape(pd, &len)) {
-    fatal("Parse error on line %d of nmap-services-probes: bad probe string escaping", lineno);
+    fatal("Parse error on line %d of nmap-service-probes: bad probe string escaping", lineno);
   }
   setProbeString((const u8 *)pd, len);
 }
@@ -878,12 +878,12 @@ void ServiceProbe::setPortVector(vector<u16> *portv, const char *portstr,
     if (isdigit((int) *current_range)) {
       rangestart = strtol(current_range, &endptr, 10);
       if (rangestart < 0 || rangestart > 65535) {
-	fatal("Parse error on line %d of nmap-services-probes: Ports must be between 0 and 65535 inclusive", lineno);
+	fatal("Parse error on line %d of nmap-service-probes: Ports must be between 0 and 65535 inclusive", lineno);
       }
       current_range = endptr;
       while(isspace((int) *current_range)) current_range++;
     } else {
-      fatal("Parse error on line %d of nmap-services-probes: An example of proper portlist form is \"21-25,53,80\"", lineno);
+      fatal("Parse error on line %d of nmap-service-probes: An example of proper portlist form is \"21-25,53,80\"", lineno);
     }
 
     /* Now I have a rangestart, time to go after rangeend */
@@ -895,14 +895,14 @@ void ServiceProbe::setPortVector(vector<u16> *portv, const char *portstr,
       if (isdigit((int) *current_range)) {
 	rangeend = strtol(current_range, &endptr, 10);
 	if (rangeend < 0 || rangeend > 65535 || rangeend < rangestart) {
-	  fatal("Parse error on line %d of nmap-services-probes: Ports must be between 0 and 65535 inclusive", lineno);
+	  fatal("Parse error on line %d of nmap-service-probes: Ports must be between 0 and 65535 inclusive", lineno);
 	}
 	current_range = endptr;
       } else {
-	fatal("Parse error on line %d of nmap-services-probes: An example of proper portlist form is \"21-25,53,80\"", lineno);
+	fatal("Parse error on line %d of nmap-service-probes: An example of proper portlist form is \"21-25,53,80\"", lineno);
       }
     } else {
-      fatal("Parse error on line %d of nmap-services-probes: An example of proper portlist form is \"21-25,53,80\"", lineno);
+      fatal("Parse error on line %d of nmap-service-probes: An example of proper portlist form is \"21-25,53,80\"", lineno);
     }
 
     /* Now I have a rangestart and a rangeend, so I can add these ports */
@@ -914,7 +914,7 @@ void ServiceProbe::setPortVector(vector<u16> *portv, const char *portstr,
     /* Find the next range */
     while(isspace((int) *current_range)) current_range++;
     if (*current_range && *current_range != ',') {
-      fatal("Parse error on line %d of nmap-services-probes: An example of proper portlist form is \"21-25,53,80\"", lineno);
+      fatal("Parse error on line %d of nmap-service-probes: An example of proper portlist form is \"21-25,53,80\"", lineno);
     }
     if (*current_range == ',')
       current_range++;
@@ -922,7 +922,7 @@ void ServiceProbe::setPortVector(vector<u16> *portv, const char *portstr,
 }
 
   // Takes a string as given in the 'ports '/'sslports ' line of
-  // nmap-services-probes.  Pass in the list from the appropriate
+  // nmap-service-probes.  Pass in the list from the appropriate
   // line.  For 'sslports', tunnel should be specified as
   // SERVICE_TUNNEL_SSL.  Otherwise use SERVICE_TUNNEL_NONE.  The line
   // number is requested because this function will bail with an error
@@ -1337,7 +1337,7 @@ bool dropdown = false;
  if (probe_state == PROBESTATE_NONMATCHINGPROBES) {
    if (!dropdown && current_probe != AP->probes.end()) current_probe++;
    while (current_probe != AP->probes.end()) {
-     // The protocol must be right, it must be a nonmatching port ('cause we did thos),
+     // The protocol must be right, it must be a nonmatching port ('cause we did those),
      // and we better either have no soft match yet, or the soft service match must
      // be available via this probe.
      if ((proto == (*current_probe)->getProbeProtocol()) && 
@@ -1475,6 +1475,18 @@ ServiceGroup::~ServiceGroup() {
   delete SPM;
 }
 
+/* Called if data is read for a service or a TCP connection made.  If
+   the port state is currently PORT_UNFILTERED, changes to
+   PORT_OPEN. */
+static void adjustPortStateIfNeccessary(ServiceNFO *svc) {
+
+  if (svc->port->state == PORT_OPENFILTERED) {
+    svc->target->ports.addPort(svc->portno, svc->proto, NULL, PORT_OPEN);
+  }
+
+  return;
+}
+
   // Sends probe text to an open connection.  In the case of a NULL probe, there
   // may be no probe text
   static int send_probe_text(nsock_pool nsp, nsock_iod nsi, ServiceNFO *svc,
@@ -1661,8 +1673,19 @@ void end_svcprobe(nsock_pool nsp, enum serviceprobestate probe_state, ServiceGro
   svc->probe_state = probe_state;
   member = find(SG->services_in_progress.begin(), SG->services_in_progress.end(),
 		  svc);
-  assert(*member);
-  SG->services_in_progress.erase(member);
+  if (member != SG->services_in_progress.end()) {
+    assert(*member == svc);
+    SG->services_in_progress.erase(member);
+  } else {
+    /* A probe can finish from services_remaining if the host times out before the
+       probe has even started */
+    member = find(SG->services_remaining.begin(), SG->services_remaining.end(),
+		  svc);
+    assert(member != SG->services_remaining.end());
+    assert(*member == svc);
+    SG->services_remaining.erase(member);
+  }
+
   SG->services_finished.push_back(svc);
 
   considerPrintingStats(SG);
@@ -1705,6 +1728,10 @@ void servicescan_connect_handler(nsock_pool nsp, nsock_event nse, void *mydata)
     }
 #endif
 
+    /* If the port is TCP, it is now known to be open rather than openfiltered */
+    if (svc->proto == IPPROTO_TCP)
+      adjustPortStateIfNeccessary(svc);
+
     // Yeah!  Connection made to the port.  Send the appropriate probe
     // text (if any is needed -- might be NULL probe)
     svc->currentprobe_exec_time = *nsock_gettimeofday();
@@ -1773,20 +1800,6 @@ void servicescan_write_handler(nsock_pool nsp, nsock_event nse, void *mydata) {
   return;
 }
 
-/* Called if data is read for a service.  Checks if the service is UDP and
-   the port is in state PORT_OPENFILTERED.  If that is the case, the state
-   is changed to PORT_OPEN, as only an open port would return a response */
-static void adjustPortStateIfNeccessary(ServiceNFO *svc) {
-  if (svc->proto != IPPROTO_UDP)
-    return;
-
-  if (svc->port->state == PORT_OPENFILTERED) {
-    svc->target->ports.addPort(svc->portno, IPPROTO_UDP, NULL, PORT_OPEN);
-  }
-
-  return;
-}
-
 void servicescan_read_handler(nsock_pool nsp, nsock_event nse, void *mydata) {
   nsock_iod nsi = nse_iod(nse);
   enum nse_status status = nse_status(nse);
@@ -1806,7 +1819,7 @@ void servicescan_read_handler(nsock_pool nsp, nsock_event nse, void *mydata) {
   } else if (status == NSE_STATUS_SUCCESS) {
     // w00p, w00p, we read something back from the port.
     readstr = (u8 *) nse_readbuf(nse, &readstrlen);
-    adjustPortStateIfNeccessary(svc); /* A UDP response means PORT_OPENFILTERED is really PORT_OPEN */
+    adjustPortStateIfNeccessary(svc); /* A response means PORT_OPENFILTERED is really PORT_OPEN */
     svc->appendtocurrentproberesponse(readstr, readstrlen);
     // now get the full version
     readstr = svc->getcurrentproberesponse(&readstrlen);
@@ -1946,7 +1959,7 @@ void servicescan_read_handler(nsock_pool nsp, nsock_event nse, void *mydata) {
   } else {
     fatal("Unexpected status (%d) in NSE_TYPE_READ callback.", (int) status);
   }
-
+  
   // We may have room for more pr0bes!
   launchSomeServiceProbes(nsp, SG);
   return;
