@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: tcpip.cc,v 1.47 2004/10/18 16:59:37 fyodor Exp $ */
+/* $Id: tcpip.cc,v 1.52 2005/02/05 22:37:55 fyodor Exp $ */
 
 
 #include "tcpip.h"
@@ -139,6 +139,7 @@ void nmapwin_list_interfaces();
 int if2nameindex(int ifi);
 #endif
 
+static PacketCounter PC;
 
 #ifndef WIN32 /* Already defined in wintcpip.c for now */
 void sethdrinclude(int sd) {
@@ -171,6 +172,40 @@ const char *proto2ascii(u8 proto, bool uppercase) {
 
 }
 
+static char *ll2shortascii(unsigned long long bytes, char *buf, int buflen) {
+  if (buflen < 2 || !buf) fatal("Bogus parameter passed to ll2shortascii");
+
+  if (bytes > 1000000) {
+    snprintf(buf, buflen, "%.3gMB", bytes / 1000000.0);
+  } else if (bytes > 10000) {
+    snprintf(buf, buflen, "%.3gKB", bytes / 1000.0);
+  } else snprintf(buf, buflen, "%uB", (unsigned int) bytes);
+    
+  return buf;
+}
+
+/* Fill buf (up to buflen -- truncate if necessary but always
+   terminate) with a short representation of the packet stats.
+   Returns buf.  Aborts if there is a problem. */
+char *getFinalPacketStats(char *buf, int buflen) {
+  char sendbytesasc[16], recvbytesasc[16];
+
+  if (buflen <= 10 || !buf)
+    fatal("getFinalPacketStats called with woefully inadequate parameters");
+
+  snprintf(buf, buflen, 
+#if WIN32
+	  "Raw packets sent: %I64u (%s) | Rcvd: %I64u (%s)",
+#else
+	  "Raw packets sent: %llu (%s) | Rcvd: %llu (%s)",
+#endif
+	   PC.sendPackets,
+	   ll2shortascii(PC.sendBytes, sendbytesasc, sizeof(sendbytesasc)),
+	   PC.recvPackets,
+	   ll2shortascii(PC.recvBytes, recvbytesasc, sizeof(recvbytesasc)));
+  return buf;
+}
+
 
   /* Takes an IP PACKET and prints it if packet tracing is enabled.
      'packet' must point to the IPv4 header. The direction must be
@@ -181,6 +216,14 @@ void PacketTrace::trace(pdirection pdir, const u8 *packet, u32 len,
 			struct timeval *now) {
   struct timeval tv;
 
+  if (pdir == SENT) {
+    PC.sendPackets++;
+    PC.sendBytes += len;
+  } else {
+    PC.recvPackets++;
+    PC.recvBytes += len;
+  }
+
   if (!o.packetTrace()) return;
 
   if (now)
@@ -317,7 +360,7 @@ const char *ippackethdrinfo(const u8 *packet, u32 len) {
   char srchost[INET6_ADDRSTRLEN], dsthost[INET6_ADDRSTRLEN];
   char *p;
   struct in_addr saddr, daddr;
-  int frag_off = 0;
+  int frag_off = 0, more_fragments = 0;
   char fragnfo[64] = "";
   char tflags[10];
   if (ip->ip_v != 4)
@@ -332,10 +375,12 @@ const char *ippackethdrinfo(const u8 *packet, u32 len) {
   inet_ntop(AF_INET, &saddr, srchost, sizeof(srchost));
   inet_ntop(AF_INET, &daddr, dsthost, sizeof(dsthost));
 
-  frag_off = BSDFIX(ntohs(ip->ip_off) & 8191 /* 2^13 - 1 */);
-  if (frag_off) {
-    snprintf(fragnfo, sizeof(fragnfo), " frag offset=%d)", frag_off);
+  frag_off = 8 * (BSDUFIX(ip->ip_off) & 8191) /* 2^13 - 1 */;
+  more_fragments = BSDUFIX(ip->ip_off) & IP_MF;
+  if (frag_off || more_fragments) {
+    snprintf(fragnfo, sizeof(fragnfo), " frag offset=%d%s", frag_off, more_fragments ? "+" : "");
   }
+  
 
   snprintf(ipinfo, sizeof(ipinfo), "ttl=%d id=%d iplen=%d%s", 
 	   ip->ip_ttl, ntohs(ip->ip_id), BSDUFIX(ip->ip_len), fragnfo);
@@ -344,9 +389,37 @@ const char *ippackethdrinfo(const u8 *packet, u32 len) {
     char tcpinfo[64] = "";
     char buf[32];
     tcp = (struct tcphdr *)  (packet + ip->ip_hl * 4);
-    if ((len < (u32) ip->ip_hl * 4 + 20) || len < (u32) ( ip->ip_hl + tcp->th_off) * 4)
-      Strncpy(protoinfo, "TCP header incomplete", sizeof(protoinfo));
-    else {
+    if (frag_off > 8 || len < (u32) ip->ip_hl * 4 + 8) 
+      snprintf(protoinfo, sizeof(protoinfo), "TCP %s:?? > %s:?? ?? %s (incomplete)", srchost, dsthost, ipinfo);
+    else if (frag_off == 8 && len >= (u32) ip->ip_hl * 4 + 8) {// we can get TCP flags nad ACKn
+      tcp = (struct tcphdr *)((u8 *) tcp - frag_off); // ugly?
+      p = tflags;
+      /* These are basically in tcpdump order */
+      if (tcp->th_flags & TH_SYN) *p++ = 'S';
+      if (tcp->th_flags & TH_FIN) *p++ = 'F';
+      if (tcp->th_flags & TH_RST) *p++ = 'R';
+      if (tcp->th_flags & TH_PUSH) *p++ = 'P';
+      if (tcp->th_flags & TH_ACK) {
+	*p++ = 'A';
+	snprintf(tcpinfo, sizeof(tcpinfo), " ack=%lu", 
+		 (unsigned long) ntohl(tcp->th_ack));
+      }
+      if (tcp->th_flags & TH_URG) *p++ = 'U';
+      if (tcp->th_flags & TH_ECE) *p++ = 'E'; /* rfc 2481/3168 */
+      if (tcp->th_flags & TH_CWR) *p++ = 'C'; /* rfc 2481/3168 */
+      *p++ = '\0';
+
+      snprintf(protoinfo, sizeof(protoinfo), "TCP %s:?? > %s:?? %s %s %s",
+	       srchost, dsthost, tflags, ipinfo, tcpinfo);
+    } else if (len < (u32) ip->ip_hl * 4 + 16) { // we can get ports an seq
+      snprintf(tcpinfo, sizeof(tcpinfo), "seq=%lu (incomplete)", (unsigned long) ntohl(tcp->th_seq));
+      snprintf(protoinfo, sizeof(protoinfo), "TCP %s:%d > %s:%d ?? %s %s",
+	       srchost, ntohs(tcp->th_sport), dsthost, ntohs(tcp->th_dport), ipinfo, tcpinfo);
+    } else if (len < (u32) ip->ip_hl * 4 + 16 && !frag_off) { // we can't get TCP flags
+      snprintf(protoinfo, sizeof(protoinfo), "TCP %s:%d > %s:%d ?? %s %s (incomplete)",
+	       srchost, ntohs(tcp->th_sport), dsthost, ntohs(tcp->th_dport),
+	       ipinfo, tcpinfo);
+    } else { // at least first 16 bytes of TCP header are there (everything we need)
 
       snprintf(tcpinfo, sizeof(tcpinfo), "seq=%lu win=%hi", 
 	       (unsigned long) ntohl(tcp->th_seq),
@@ -372,12 +445,16 @@ const char *ippackethdrinfo(const u8 *packet, u32 len) {
 	       srchost, ntohs(tcp->th_sport), dsthost, ntohs(tcp->th_dport),
 	       tflags, ipinfo, tcpinfo);
     }
+  } else if (ip->ip_p == IPPROTO_UDP && frag_off) {
+      snprintf(protoinfo, sizeof(protoinfo), "UDP %s:?? > %s:?? fragment %s (incomplete)", srchost, dsthost, ipinfo);
   } else if (ip->ip_p == IPPROTO_UDP) {
     udp =  (udphdr_bsd *) (packet + sizeof(struct ip));
 
     snprintf(protoinfo, sizeof(protoinfo), "UDP %s:%d > %s:%d %s",
 	     srchost, ntohs(udp->uh_sport), dsthost, ntohs(udp->uh_dport),
 	     ipinfo);
+  } else if (ip->ip_p == IPPROTO_ICMP && frag_off) {
+      snprintf(protoinfo, sizeof(protoinfo), "ICMP %s > %s fragment %s (incomplete)", srchost, dsthost, ipinfo);
   } else if (ip->ip_p == IPPROTO_ICMP) {
     char icmptype[128];
     struct ppkt {
@@ -479,7 +556,8 @@ const char *ippackethdrinfo(const u8 *packet, u32 len) {
     snprintf(protoinfo, sizeof(protoinfo), "ICMP %s > %s %s (type=%d/code=%d) %s",
 	     srchost, dsthost, icmptype, ping->type, ping->code, ipinfo);
   } else {
-    snprintf(protoinfo, sizeof(protoinfo), "Unknown protocol: %s", ipinfo);
+    snprintf(protoinfo, sizeof(protoinfo), "Unknown protocol (%d): %s", 
+	     ip->ip_p, ipinfo);
   }    
 
   return protoinfo;
@@ -661,7 +739,7 @@ assert(victim);
 assert(source);
 
 if (optlen % 4) {
-  fatal("send_tcp_raw called with an option length argument of %d which is illegal because it is not divisible by 4", optlen);
+  fatal("build_tcp_raw() called with an option length argument of %d which is illegal because it is not divisible by 4", optlen);
 }
 
 /* Time to live */
@@ -776,6 +854,10 @@ int send_ip_packet(int sd, u8 *packet, unsigned int packetlen) {
   assert(packet);
   assert( (int) packetlen > 0);
 
+  // fragmentation requested && packet is bigger than MTU
+  if (o.fragscan && ( packetlen - ip->ip_hl * 4 > (unsigned int) o.fragscan ))
+      return send_frag_ip_packet(sd, packet, packetlen, o.fragscan);
+
   memset(&sock, 0, sizeof(sock));
   sock.sin_family = AF_INET;
 #if HAVE_SOCKADDR_SA_LEN
@@ -793,7 +875,7 @@ int send_ip_packet(int sd, u8 *packet, unsigned int packetlen) {
       sock.sin_port = udp->uh_dport;
     }
   }
-  /* I'll try leaving out port and est address and see what happens */
+  /* I'll try leaving out dest port and address and see what happens */
   /* sock.sin_port = htons(dport);
      sock.sin_addr.s_addr = victim->s_addr; */
   
@@ -802,6 +884,108 @@ int send_ip_packet(int sd, u8 *packet, unsigned int packetlen) {
   return res;
 }
 
+/* Create and send all fragments of a pre-built IPv4 packet
+ * Minimal MTU for IPv4 is 68 and maximal IPv4 header size is 60
+ * which gives us a right to cut TCP header after 8th byte
+ * (shouldn't we inflate the header to 60 bytes too?) */
+int send_frag_ip_packet(int sd, u8 *packet, unsigned int packetlen, unsigned int mtu)
+{
+    struct ip *ip = (struct ip *) packet;
+    int headerlen = ip->ip_hl * 4; // better than sizeof(struct ip)
+    unsigned int datalen = packetlen - headerlen;
+    int fdatalen = 0, res = 0;
+
+    assert(headerlen <= (int) packetlen);
+    assert(headerlen >= 20 && headerlen <= 60); // sanity check (RFC791)
+    assert(mtu > 0 && mtu % 8 == 0); // otherwise, we couldn't set Fragment offset (ip->ip_off) correctly
+
+    if (datalen <= mtu) {
+        error("Warning: fragmentation (mtu=%i) requested but the payload is too small already (%i)", mtu, datalen);
+        return send_ip_packet(sd, packet, packetlen);
+    }
+
+    u8 *fpacket = (u8 *) safe_malloc(headerlen + mtu);
+    memcpy(fpacket, packet, headerlen + mtu);
+    ip = (struct ip *) fpacket;
+
+    // create fragments and send them
+    for (int fragment = 1; fragment * mtu < datalen + mtu; fragment++) {
+        fdatalen = (fragment * mtu <= datalen ? mtu : datalen % mtu);
+        ip->ip_len = BSDFIX(headerlen + fdatalen);
+        ip->ip_off = BSDFIX((fragment-1) * mtu / 8);
+        if ((fragment-1) * mtu + fdatalen < datalen)
+            ip->ip_off |= BSDFIX(IP_MF);
+#if HAVE_IP_IP_SUM
+        ip->ip_sum = in_cksum((unsigned short *)ip, headerlen);
+#endif
+        if (fragment > 1) // copy data payload
+            memcpy(fpacket + headerlen, packet + headerlen + (fragment - 1) * mtu, fdatalen);
+        res = send_ip_packet(sd, fpacket, headerlen + fdatalen);
+        if (res == -1)
+            break;
+    }
+
+    free(fpacket);
+
+    return res;
+}
+
+/* Builds an ICMP packet (including an IP header) by packing the fields
+   with the given information.  It allocates a new buffer to store the
+   packet contents, and then returns that buffer.  The packet is not
+   actually sent by this function.  Caller must delete the buffer when
+   finished with the packet.  The packet length is returned in
+   packetlen, which must be a valid int pointer. */
+u8 *build_icmp_raw(const struct in_addr *source, const struct in_addr *victim, 
+		   int ttl, u16 ipid, u16 seq, unsigned short id, u8 ptype, 
+		   u8 pcode, char *data, u16 datalen, u32 *packetlen) {
+
+struct ppkt {
+  u8 type;
+  u8 code;
+  u16 checksum;
+  u16 id;
+  u16 seq;
+  u8 data[1500]; /* Note -- first 4-12 bytes can be used for ICMP header */
+} pingpkt;
+u32 *datastart = (u32 *) pingpkt.data;
+int dlen = sizeof(pingpkt.data); 
+int icmplen=0;
+char *ping = (char *) &pingpkt;
+
+ pingpkt.type = ptype;
+ pingpkt.code = pcode;
+
+ if (ptype == 8 && pcode == 0) /* echo request */ {
+   icmplen = 8;
+ } else if (ptype == 13 && pcode == 0) /* ICMP timestamp req */ {
+   icmplen = 20;
+   memset(datastart, 0, 12);
+   datastart += 12;
+   datalen -= 12;
+ } else if (ptype == 17 && pcode == 0) /* icmp netmask req */ {
+   icmplen = 12;
+   *datastart++ = 0;
+   datalen -= 4;
+ } else 
+   fatal("Unknown icmp type/code (%d/%d) in build_icmp_raw", ptype, pcode);
+
+ if (datalen > 0) {
+   icmplen += MIN(dlen, datalen);
+   memset(datastart, 0, MIN(dlen, datalen));
+ }
+/* Fill out the ping packet */
+
+pingpkt.code = 0;
+pingpkt.id = id;
+pingpkt.seq = seq;
+pingpkt.checksum = 0;
+pingpkt.checksum = in_cksum((unsigned short *)ping, icmplen);
+
+return build_ip_raw(source, victim, o.ttl, IPPROTO_ICMP, get_random_u16(),
+		    ping, icmplen, packetlen);
+}
+
 void readippacket(const u8 *packet, int readdata) {
   struct ip *ip = (struct ip *) packet;
   switch(ip->ip_p) {
@@ -1042,183 +1226,13 @@ int send_udp_raw( int sd, struct in_addr *source, const struct in_addr *victim,
   return res;
 }
 
-int send_small_fragz_decoys(int sd, const struct in_addr *victim, u32 seq,
-			    int ttl, u16 sport, u16 dport, int flags) {
-  int decoy;
-
-  for(decoy = 0; decoy < o.numdecoys; decoy++) 
-    if (send_small_fragz(sd, &o.decoys[decoy], victim, seq, ttl, sport, 
-				dport, 
-				flags) == -1)
-      return -1;
-
-  return 0;
-}
-/* Much of this is swiped from my send_tcp_raw function above, which 
-   doesn't support fragmentation */
-int send_small_fragz(int sd, struct in_addr *source, 
-		     const struct in_addr *victim, u32 seq, int ttl,
-		     u16 sport, u16 dport, int flags)
- {
-
-struct pseudo_header { 
-/*for computing TCP checksum, see TCP/IP Illustrated p. 145 */
-  u32 s_addy;
-  u32 d_addr;
-  u8 zer0;
-  u8 protocol;
-  u16 length;
-};
-/*In this placement we get data and some field alignment so we aren't wasting
-  too much to compute the TCP checksum.*/
-
-unsigned char packet[sizeof(struct ip) + sizeof(struct tcphdr) + 100];
-struct ip *ip = (struct ip *) packet;
-struct tcphdr *tcp = (struct tcphdr *) (packet + sizeof(struct ip));
-struct pseudo_header *pseudo = (struct pseudo_header *) (packet + sizeof(struct ip) - sizeof(struct pseudo_header)); 
-unsigned char *frag2 = packet + sizeof(struct ip) + 16;
-struct ip *ip2 = (struct ip *) (frag2 - sizeof(struct ip));
-static int myttl = 0;
-int res;
-struct sockaddr_in sock;
-int id;
-
-assert(source);
-
-/* Time to live */
-if (ttl == -1) {
-  myttl = (time(NULL) % 14) + 51;
-} else {
-  myttl = ttl;
-}
-
-#ifndef WIN32
-/* It was a tough decision whether to do this here for every packet
-   or let the calling function deal with it.  In the end I grudgingly decided
-   to do it here and potentially waste a couple microseconds... */
-sethdrinclude(sd);
-#endif
-
-/*Why do we have to fill out this damn thing? This is a raw packet, after all */
-sock.sin_family = AF_INET;
-sock.sin_port = htons(dport);
-
-sock.sin_addr.s_addr = victim->s_addr;
-
-memset((char *)packet, 0, sizeof(struct ip) + sizeof(struct tcphdr));
-
-pseudo->s_addy = source->s_addr;
-pseudo->d_addr = victim->s_addr;
-pseudo->protocol = IPPROTO_TCP;
-pseudo->length = htons(sizeof(struct tcphdr));
-
-tcp->th_sport = htons(sport);
-tcp->th_dport = htons(dport);
-tcp->th_seq = (seq)? htonl(seq) : get_random_uint();
-
-tcp->th_off = 5 /*words*/;
-tcp->th_flags = flags;
-
-tcp->th_win = htons(2048); /* Who cares */
-
-tcp->th_sum = in_cksum((unsigned short *)pseudo, 
-		       sizeof(struct tcphdr) + sizeof(struct pseudo_header));
-
-/* Now for the ip header of frag1 */
-
-memset((char *) packet, 0, sizeof(struct ip)); 
-ip->ip_v = 4;
-ip->ip_hl = 5;
-/*RFC 791 allows 8 octet frags, but I get "operation not permitted" (EPERM)
-  when I try that.  */
-ip->ip_len = BSDFIX(sizeof(struct ip) + 16);
-id = ip->ip_id = get_random_uint();
-ip->ip_off = BSDFIX(MORE_FRAGMENTS);
-ip->ip_ttl = myttl;
-ip->ip_p = IPPROTO_TCP;
-ip->ip_src.s_addr = source->s_addr;
-ip->ip_dst.s_addr = victim->s_addr;
-
-#if HAVE_IP_IP_SUM
-ip->ip_sum= in_cksum((unsigned short *)ip, sizeof(struct ip));
-#endif
-if (o.debugging > 1) {
-  log_write(LOG_STDOUT, "Raw TCP packet fragment #1 creation completed!  Here it is:\n");
-  hdump(packet,20);
-}
-if (o.debugging > 1) 
-  log_write(LOG_STDOUT, "\nTrying sendto(%d , packet, %d, 0 , %s , %d)\n",
-	 sd, ntohs(ip->ip_len), inet_ntoa(*victim),
-	 (int) sizeof(struct sockaddr_in));
-
-if ((res = Sendto("send_small_fragz", sd, packet,sizeof(struct ip) + 16 , 0, (struct sockaddr *)&sock, sizeof(struct sockaddr_in))) == -1)
-  {
-    perror("sendto in send_syn_fragz");
-    return -1;
-  }
-if (o.debugging > 1) log_write(LOG_STDOUT, "successfully sent %d bytes of raw_tcp!\n", res);
-
-/* Create the second fragment */
-
-memset((char *) ip2, 0, sizeof(struct ip));
-ip2->ip_v= 4;
-ip2->ip_hl = 5;
-ip2->ip_len = BSDFIX(sizeof(struct ip) + 4); /* the rest of our TCP packet */
-ip2->ip_id = id;
-ip2->ip_off = BSDFIX(2);
-ip2->ip_ttl = myttl;
-ip2->ip_p = IPPROTO_TCP;
-ip2->ip_src.s_addr = source->s_addr;
-#ifdef WIN32
-// I'm not sure exactly why this is neccessary --Fyodor
- if(source->s_addr == victim->s_addr) ip2->ip_src.s_addr++;
-#endif
-ip2->ip_dst.s_addr = victim->s_addr;
-
-#if HAVE_IP_IP_SUM
-ip2->ip_sum = in_cksum((unsigned short *)ip2, sizeof(struct ip));
-#endif
-if (o.debugging > 1) {
-  log_write(LOG_STDOUT, "Raw TCP packet fragment creation completed!  Here it is:\n");
-  hdump(packet,20);
-}
-if (o.debugging > 1) 
-
-  log_write(LOG_STDOUT, "\nTrying sendto(%d , ip2, %d, 0 , %s , %d)\n", sd, 
-	 ntohs(ip2->ip_len), inet_ntoa(*victim), (int) sizeof(struct sockaddr_in));
-if ((res = Sendto("send_small_fragz", sd, (const unsigned char *) ip2,
-		  sizeof(struct ip) + 4 , 0,  (struct sockaddr *)&sock, 
-		  (int) sizeof(struct sockaddr_in))) == -1)
-  {
-    perror("sendto in send_tcp_raw frag #2");
-    return -1;
-  }
-
-return 1;
-}
-
-int send_ip_raw_decoys( int sd, const struct in_addr *victim, int ttl,
-			u8 proto, char *data, u16 datalen) {
-
-  int decoy;
-
-  for(decoy = 0; decoy < o.numdecoys; decoy++) 
-    if (send_ip_raw(sd, &o.decoys[decoy], victim, ttl, proto, data, 
-			   datalen) == -1)
-      return -1;
-
-  return 0;
-
-
-}
-
 /* Builds an IP packet (including an IP header) by packing the fields
    with the given information.  It allocates a new buffer to store the
    packet contents, and then returns that buffer.  The packet is not
    actually sent by this function.  Caller must delete the buffer when
    finished with the packet.  The packet length is returned in
    packetlen, which must be a valid int pointer. */
-u8 *build_ip_raw(struct in_addr *source, const struct in_addr *victim, 
+u8 *build_ip_raw(const struct in_addr *source, const struct in_addr *victim, 
 		 int ttl, u8 proto, u16 ipid, char *data, u16 datalen, 
 		 u32 *packetlen) 
 {
@@ -1623,7 +1637,8 @@ if (!pd) fatal("NULL packet device passed to readip_pcap");
      }
    }
  } while(!timedout && (!p || (*p & 0x40) != 0x40)); /* Go until we get IPv4 packet */
- if (timedout) {
+ 
+if (timedout) {
    *len = 0;
    return NULL;
  }
@@ -1742,13 +1757,13 @@ void set_pcap_filter(Target *target,
   if (vsnprintf(buf, sizeof(buf), bpf, ap) >= (int) sizeof(buf))
     fatal("set_pcap_filter called with too-large filter arg\n");
   va_end(ap);
-  
-  if (o.debugging)
-    log_write(LOG_STDOUT, "Packet capture filter (device %s): %s\n", target->device, buf);
-  
+
   /* Due to apparent bug in libpcap */
   if (islocalhost(target->v4hostip()))
     buf[0] = '\0';
+
+  if (o.debugging)
+    log_write(LOG_STDOUT, "Packet capture filter (device %s): %s\n", target->device, buf);
   
   if (pcap_compile(pd, &fcode, buf, 0, netmask) < 0)
     fatal("Error compiling our pcap filter: %s\n", pcap_geterr(pd));
@@ -2436,3 +2451,4 @@ int IPProbe::storePacket(u8 *ippacket, u32 len) {
   }
   return 0;
 }
+
